@@ -216,7 +216,7 @@ class CellEvaluator(bpopt.evaluators.Evaluator):
 
         return self.objective_list(obj_dict)
 
-    def set_neuron_variables_and_evaluate_with_lists(
+    def init_simulator_and_evaluate_with_lists(
         self, param_list=None, target='scores'
     ):
         """Set NEURON variables and run evaluation with lists.
@@ -224,7 +224,7 @@ class CellEvaluator(bpopt.evaluators.Evaluator):
         Setting the NEURON variables is necessary when using ipyparallel,
         since the new subprocesses have pristine NEURON.
         """
-        self.sim.set_neuron_variables()
+        self.sim.initialize()
         return self.evaluate_with_lists(param_list=param_list, target=target)
 
     def evaluate(self, param_list=None, target='scores'):
