@@ -8,6 +8,10 @@ import imp
 import ctypes
 import platform
 import warnings
+import pathlib
+
+from bluepyopt.ephys.acc import arbor
+
 
 logger = logging.getLogger(__name__)
 
@@ -47,7 +51,7 @@ class NrnSimulator(object):
 
         self.cvode_active = cvode_active
 
-        self.set_neuron_variables()
+        self.initialize()
 
         self.random123_globalindex = random123_globalindex
 
@@ -104,8 +108,8 @@ class NrnSimulator(object):
 
         return neuron
 
-    def set_neuron_variables(self):
-        """Set Neuron variables"""
+    def initialize(self):
+        """Initialize simulator: Set Neuron variables"""
         self.neuron.h.load_file('stdrun.hoc')
         self.neuron.h.dt = self.dt
         self.neuron.h.cvode_active(1 if self.cvode_active else 0)
@@ -280,3 +284,94 @@ class LFPySimulatorException(Exception):
 
         super(LFPySimulatorException, self).__init__(message)
         self.original = original
+
+
+class ArbSimulator(object):
+
+    """Arbor simulator"""
+
+    def __init__(self, dt=None, ext_catalogues=None):
+        """Constructor
+
+        Args:
+            dt (float): the integration time step used by Arbor.
+            ext_catalogues (): Name to path mapping of non-Arbor built-in
+            NMODL mechanism catalogues compiled with modcc
+        """
+
+        self.dt = dt
+        self.ext_catalogues = ext_catalogues
+        if ext_catalogues is not None:
+            for cat, cat_path in ext_catalogues.items():
+                cat_lib = '%s-catalogue.so' % cat
+                cat_path = pathlib.Path(cat_path).resolve()
+                if not os.path.exists(cat_path / cat_lib):
+                    raise ArbSimulatorException(
+                        'Cannot find %s at %s - first build' % (cat_lib,
+                                                                cat_path) +
+                        ' mechanism catalogue with modcc:' +
+                        ' arbor-build-catalogue %s %s' % (cat, cat_path))
+        # TODO: add parameters for discretization
+
+    def initialize(self):
+        """Initialize simulator"""
+        pass
+
+    def instantiate(self, morph, decor, labels):
+        cable_cell = arbor.cable_cell(morphology=morph,
+                                      decor=decor,
+                                      labels=labels)
+
+        arb_cell_model = arbor.single_cell_model(cable_cell)
+
+        # Add catalogues with explicit qualifiers
+        arb_cell_model.properties.catalogue = arbor.catalogue()
+
+        # User-supplied catalogues take precedence
+        if self.ext_catalogues is not None:
+            for cat, cat_path in self.ext_catalogues.items():
+                cat_lib = '%s-catalogue.so' % cat
+                cat_path = pathlib.Path(cat_path).resolve()
+                arb_cell_model.properties.catalogue.extend(
+                    arbor.load_catalogue(cat_path / cat_lib),
+                    cat + "::")
+
+        # Built-in catalogues are always added (could be made optional)
+        if self.ext_catalogues is None or \
+                'default' not in self.ext_catalogues:
+            arb_cell_model.properties.catalogue.extend(
+                arbor.default_catalogue(), "default::")
+
+        if self.ext_catalogues is None or \
+                'BBP' not in self.ext_catalogues:
+            arb_cell_model.properties.catalogue.extend(
+                arbor.bbp_catalogue(), "BBP::")
+
+        if self.ext_catalogues is None or \
+                'allen' not in self.ext_catalogues:
+            arb_cell_model.properties.catalogue.extend(
+                arbor.allen_catalogue(), "allen::")
+
+        return arb_cell_model
+
+    def run(self,
+            arb_cell_model,
+            tstop=None,
+            dt=None):
+
+        dt = dt if dt is not None else self.dt
+
+        if dt is not None:
+            return arb_cell_model.run(tfinal=tstop, dt=dt)
+        else:
+            return arb_cell_model.run(tfinal=tstop)
+
+
+class ArbSimulatorException(Exception):
+
+    """All exception generated by Arbor simulator"""
+
+    def __init__(self, message):
+        """Constructor"""
+
+        super(ArbSimulatorException, self).__init__(message)
